@@ -12,6 +12,7 @@ __all__ = ('Parameter',)
 from ._feature import Feature
 from ..utils import format_title
 import biosteam as bst
+from chaospy import distributions as shape
 from inspect import signature
 
 class Parameter(Feature):
@@ -37,38 +38,46 @@ class Parameter(Feature):
         Baseline value of parameter.
     bounds : tuple[float, float]
         Lower and upper bounds of parameter.
-    kind : str
-        * 'design': Parameter only affects unit operation design.
-        * 'coupled': Parameter affects mass and energy balances.
-        * 'isolated': Parameter does not affect the system in any way.
+    coupled : str
+        Whether parameter is coupled to the system's mass and energy balances.
+        This allows a ConvergenceModel to predict it's impact on recycle loops.
+        Defaults to False.
     hook : Callable
         Should return the new parameter value given the sample.
-    scale : float, optional
-        The sample is multiplied by the scale before setting.
         
     """
     __slots__ = ('setter', 'system', 'distribution', 
-                 'baseline', 'bounds', 'kind', 'hook',
-                 'description', 'scale')
+                 'baseline', 'bounds', 'coupled', 'hook',
+                 'description', 'active', 'last_value')
     
     def __init__(self, name, setter, element, system, distribution,
-                 units, baseline, bounds, kind, hook, description, scale):
+                 units, baseline, bounds, coupled, hook, description):
         if not name: name, *_ = signature(setter).parameters.keys()
         super().__init__(format_title(name), units, element)
-        if kind is None: kind = 'isolated'
         self.setter = setter.setter if isinstance(setter, Parameter) else setter
         self.system = system
-        self.distribution = distribution
-        if not bounds:
+        if isinstance(distribution, str):
+            match distribution.lower():
+                case 'triangle' | 'triangular':
+                    distribution = shape.Triangle(bounds[0], baseline, bounds[1])
+                case 'uniform':
+                    distribution = shape.Uniform(*bounds)
+                case _:
+                    raise ValueError(f"invalid distribution {distribution!r}; distribution must be either 'triangular' or 'uniform'")
+        elif bounds is None:
             if distribution: bounds = (distribution.lower[0], distribution.upper[0])
-        if bounds and not baseline:
+        elif not distribution:
+            distribution = shape.Uniform(*bounds)
+        if bounds and baseline is None:
             baseline = 0.5 * (bounds[0] + bounds[1])
+        self.distribution = distribution
         self.baseline = baseline
         self.bounds = bounds
-        self.kind = kind
+        self.coupled = coupled
         self.hook = hook
         self.description = description
-        self.scale = scale
+        self.active = True
+        self.last_value = None
     
     @classmethod
     def sort_parameters(cls, parameters):
@@ -77,10 +86,11 @@ class Parameter(Feature):
             system, = set([i.system for i in parameters])
         except:
             raise ValueError('all parameters must have the same system to sort')
+        if system is None: return
         unit_path = system.units
         length = len(unit_path)
         def key(parameter):
-            if parameter.kind == 'coupled':
+            if parameter.coupled:
                 unit = parameter.unit
                 if unit: return unit_path.index(unit) 
             return length
@@ -105,23 +115,18 @@ class Parameter(Feature):
     
     def simulate(self, **dyn_sim_kwargs):
         """Simulate parameter."""
-        kind = self.kind
-        if kind in ('design', 'cost'):
-            unit = self.unit
-            if not unit: raise RuntimeError(f'no unit to run {kind} algorithm')
-            unit._reevaluate()
-        elif kind == 'coupled':
+        if self.coupled:
             subsystem = self.subsystem
-            if not subsystem: raise RuntimeError(f'no system to run {kind} algorithm')
+            if not subsystem: raise RuntimeError('no system to simulate')
             self.subsystem.simulate(**dyn_sim_kwargs)
-        elif kind == 'isolated':
-            pass
         else:
-            raise RuntimeError(f"invalid parameter kind '{kind}'")
+            unit = self.unit
+            if hasattr(unit, '_reevaluate'): unit._reevaluate()
     
     def __call__(self, value):
         if self.hook: value = self.hook(value)
-        self.setter(value if self.scale is None else value * self.scale)
+        self.setter(value)
+        self.last_value = value
         self.simulate()
     
     def _info(self):
