@@ -7,7 +7,7 @@
 # for license details.
 """
 """
-from .._graphics import mixer_graphics
+from thermosteam._graphics import mixer_graphics
 import biosteam as bst
 
 __all__ = ('BlowdownMixer',)
@@ -15,7 +15,4 @@ __all__ = ('BlowdownMixer',)
 class BlowdownMixer(bst.Facility, bst.Mixer):
     network_priority = 2
     _graphics = mixer_graphics
-    ticket_name = 'BDM'
-    def __init__(self, ID='', ins=None, outs=(), thermo=None, rigorous=False):
-        bst.Facility.__init__(self, ID, ins, outs, thermo)
-        self.rigorous = rigorous
\ No newline at end of file
+    ticket_name = 'BDM'
\ No newline at end of file
