@@ -8,7 +8,7 @@
 """
 """
 from .._unit import Unit
-from .._graphics import junction_graphics
+from thermosteam._graphics import junction_graphics
 from .._power_utility import PowerUtility
 from ..exceptions import UndefinedChemical
 from ..utils.piping import Inlets, Outlets
@@ -66,6 +66,7 @@ class Junction(Unit):
     installed_cost = 0.
     installed_costs = {}
     utility_cost = 0.
+    prioritize = False
     
     def __init__(self, ID="", upstream=None, downstream=None, thermo=None):
         self._register(ID)
