@@ -97,7 +97,7 @@ class MultiEffectEvaporator(Unit):
     >>> E1.show()
     MultiEffectEvaporator: E1
     ins...
-    [0] feed
+    [0] feed  
         phase: 'l', T: 298.15 K, P: 101325 Pa
         flow (kmol/hr): Water       55.5
                         AceticAcid  0.00833
@@ -105,18 +105,18 @@ class MultiEffectEvaporator(Unit):
                         HMF         0.000793
                         Glucose     0.555
     outs...
-    [0] solids
+    [0] solids  
         phase: 'l', T: 333.21 K, P: 20000 Pa
         flow (kmol/hr): Water       20.5
-                        AceticAcid  0.00183
-                        Furfural    5.96e-05
+                        AceticAcid  0.00181
+                        Furfural    5.29e-05
                         HMF         0.000793
                         Glucose     0.555
-    [1] liquid
+    [1] liquid  
         phase: 'l', T: 352.11 K, P: 20000 Pa
         flow (kmol/hr): Water       35
-                        AceticAcid  0.0065
-                        Furfural    0.000981
+                        AceticAcid  0.00651
+                        Furfural    0.000988
     
     >>> E1.results()
     Multi-effect evaporator                                    Units       E1
@@ -132,8 +132,8 @@ class MultiEffectEvaporator(Unit):
                         Volume                                   m^3     3.51
     Purchase cost       Evaporators (x5)                         USD 9.56e+03
                         Condenser - Double pipe                  USD 5.36e+03
-                        Vacuum system - Liquid-ring pump...      USD 1.24e+04
-    Total purchase cost                                          USD 2.73e+04
+                        Vacuum system - Liquid-ring pump...      USD 1.07e+04
+    Total purchase cost                                          USD 2.56e+04
     Utility cost                                              USD/hr     4.15
     
     Concentrate sugar setting overall vapor fraction:
@@ -151,7 +151,7 @@ class MultiEffectEvaporator(Unit):
     >>> E1.show()
     MultiEffectEvaporator: E1
     ins...
-    [0] feed
+    [0] feed  
         phase: 'l', T: 298.15 K, P: 101325 Pa
         flow (kmol/hr): Water       55.5
                         AceticAcid  0.00833
@@ -159,18 +159,18 @@ class MultiEffectEvaporator(Unit):
                         HMF         0.000793
                         Glucose     0.555
     outs...
-    [0] solids
+    [0] solids  
         phase: 'l', T: 354.91 K, P: 50892 Pa
         flow (kmol/hr): Water       50
-                        AceticAcid  0.00683
-                        Furfural    0.000535
+                        AceticAcid  0.0069
+                        Furfural    0.000577
                         HMF         0.000793
                         Glucose     0.555
-    [1] liquid
-        phase: 'l', T: 361.13 K, P: 50892 Pa
+    [1] liquid  
+        phase: 'l', T: 361.12 K, P: 50892 Pa
         flow (kmol/hr): Water       5.55
-                        AceticAcid  0.0015
-                        Furfural    0.000506
+                        AceticAcid  0.00143
+                        Furfural    0.000464
     
     >>> E1.results()
     Multi-effect evaporator                                    Units        E1
@@ -186,8 +186,8 @@ class MultiEffectEvaporator(Unit):
                         Volume                                   m^3      3.07
     Purchase cost       Evaporators (x3)                         USD  2.74e+03
                         Condenser - Double pipe                  USD   3.9e+03
-                        Vacuum system - Liquid-ring pump...      USD  1.24e+04
-    Total purchase cost                                          USD  1.91e+04
+                        Vacuum system - Liquid-ring pump...      USD  1.08e+04
+    Total purchase cost                                          USD  1.74e+04
     Utility cost                                              USD/hr      2.85
     
     Concentrate sugar setting overall vapor fraction:
@@ -205,7 +205,7 @@ class MultiEffectEvaporator(Unit):
     >>> E1.show()
     MultiEffectEvaporator: E1
     ins...
-    [0] feed
+    [0] feed  
         phase: 'l', T: 298.15 K, P: 101325 Pa
         flow (kmol/hr): Water       55.5
                         AceticAcid  0.00833
@@ -213,18 +213,18 @@ class MultiEffectEvaporator(Unit):
                         HMF         0.000793
                         Glucose     0.555
     outs...
-    [0] solids
+    [0] solids  
         phase: 'l', T: 354.91 K, P: 50892 Pa
         flow (kmol/hr): Water       50
-                        AceticAcid  0.00683
-                        Furfural    0.000535
+                        AceticAcid  0.0069
+                        Furfural    0.000577
                         HMF         0.000793
                         Glucose     0.555
-    [1] liquid
-        phase: 'l', T: 361.13 K, P: 50892 Pa
+    [1] liquid  
+        phase: 'l', T: 361.12 K, P: 50892 Pa
         flow (kmol/hr): Water       5.55
-                        AceticAcid  0.0015
-                        Furfural    0.000506
+                        AceticAcid  0.00143
+                        Furfural    0.000464
     
     >>> E1.results()
     Multi-effect evaporator                                    Units        E1
@@ -240,12 +240,13 @@ class MultiEffectEvaporator(Unit):
                         Volume                                   m^3      3.07
     Purchase cost       Evaporators (x3)                         USD  2.74e+03
                         Condenser - Double pipe                  USD   3.9e+03
-                        Vacuum system - Liquid-ring pump...      USD  1.24e+04
-    Total purchase cost                                          USD  1.91e+04
+                        Vacuum system - Liquid-ring pump...      USD  1.08e+04
+    Total purchase cost                                          USD  1.74e+04
     Utility cost                                              USD/hr      2.85
     
     """
     line = 'Multi-effect evaporator'
+    vacuum_system_preference = 'Liquid-ring pump'
     auxiliary_unit_names = ('condenser', 'mixer', 'vacuum_system', 'evaporators')
     _units = {'Area': 'm^2',
               'Volume': 'm^3'}
@@ -290,9 +291,8 @@ class MultiEffectEvaporator(Unit):
         else:
             raise ValueError("V_definition must be either 'Overall', 'First-effect', or 'First-effect duty'")
 
-    def __init__(self, ID='', ins=None, outs=(), thermo=None, *, P, V, V_definition='Overall',
-                 flash=True, chemical='7732-18-5'):
-        Unit.__init__(self, ID, ins, outs, thermo)
+    def _init(self, P, V, V_definition='Overall',
+              flash=True, chemical='7732-18-5'):
         self.P = P #: tuple[float] Pressures describing each evaporator (Pa).
         self.V = V #: [float] Molar fraction evaporated.
         self.V_definition = V_definition
@@ -311,13 +311,13 @@ class MultiEffectEvaporator(Unit):
             evaporator = self.auxiliary(
                 'evaporators', Flash,
                 ins=self.ins, 
-                outs=(None, None), P=P[0],
+                outs=(None, self.outs[0] if n == 1 else None), P=P[0],
             )
         else:
             evaporator = self.auxiliary(
                 'evaporators', Evaporator,
                 ins=self.ins,
-                outs=(None, None, None), P=P[0],
+                outs=(None, self.outs[0] if n == 1 else None), P=P[0],
                 chemical=self.chemical,
             )
         for i in range(1, n):
@@ -489,7 +489,7 @@ class MultiEffectEvaporator(Unit):
         Cost['Evaporators'] = sum(evap_costs)
         
         self.vacuum_system = bst.VacuumSystem(
-            self, 'Steam-jet ejector', vessel_volume=total_volume, P_suction=self.outs[0].P,
+            self, self.vacuum_system_preference, vessel_volume=total_volume, P_suction=self.outs[0].P,
         )
             
         
