@@ -45,11 +45,11 @@ import biosteam as bst
 from .splitting import Splitter
 from .design_tools import CEPCI_by_year, geometry, PressureVessel
 from .decorators import cost, copy_algorithm
-from .phase_equilibrium import MultiStageEquilibrium
-from .._graphics import mixer_settler_graphics
+from .stage import MultiStageEquilibrium
+from thermosteam._graphics import mixer_settler_graphics
 from .. import Unit
-from ._flash import RatioFlash
 from thermosteam import separations as sep
+import numpy as np
 
 __all__ = (
     'LLEUnit',
@@ -86,10 +86,6 @@ class LLEUnit(bst.Unit, isabstract=True):
     efficiency=1. : float, optional
         Fraction of feed in liquid-liquid equilibrium.
         The rest of the feed is divided equally between phases.
-    cache_tolerance=1e-6 : float, optional
-        Reuse previous partition coefficients to calculate LLE when 
-        the change in molar fraction of all chemicals is below this 
-        tolerance.
     forced_split_IDs : tuple[str], optional
         IDs of component with a user defined split.
     forced_split : 1d array, optional
@@ -116,18 +112,12 @@ class LLEUnit(bst.Unit, isabstract=True):
     """
     _N_outs = 2
     
-    def __init__(self, ID='', ins=None, outs=(), thermo=None,
-                 top_chemical=None, efficiency=1.0, cache_tolerance=1e-6,
-                 forced_split_IDs=None, forced_split=None):
-        bst.Unit.__init__(self, ID, ins, outs, thermo)
+    def _init(self, top_chemical=None, efficiency=1.0, forced_split_IDs=None, forced_split=None):
         #: [str] Identifier of chemical that will be favored in the low density phase.
         self.top_chemical = top_chemical
         #: [float] Fraction of feed in liquid-liquid equilibrium.
         #: The rest of the feed is divided equally between phases.
-        self.efficiency = efficiency 
-        #: [float] The change in molar fraction of individual chemicals must be 
-        #: below this tolerance to reuse partition coefficients.
-        self.cache_tolerance = cache_tolerance
+        self.efficiency = efficiency
         #: array[float] Forced splits to 0th stream for given IDs. 
         self.forced_split = forced_split
         #: tuple[str] IDs corresponding to forced splits. 
@@ -142,6 +132,7 @@ class LLEUnit(bst.Unit, isabstract=True):
         self.top_chemical = solvent
         
     def _run(self):
+        if all([i.isempty() for i in self.ins]): return
         sep.lle(*self.ins, *self.outs, self.top_chemical, self.efficiency, self.multi_stream)
         IDs = self.forced_split_IDs
         if IDs:
@@ -190,8 +181,52 @@ class LiquidsCentrifuge(Unit, isabstract=True):
 # TODO: Remove this in favor of partition coefficients
 class LiquidsRatioCentrifuge(LiquidsCentrifuge):
     line = 'Liquids centrifuge'
-    __init__ = RatioFlash.__init__
-    _run = RatioFlash._run
+    
+    def _init(self, 
+            K_chemicals, Ks, top_solvents=(), top_split=(),
+            bot_solvents=(), bot_split=()
+        ):
+        self._load_components()
+        self.K_chemicals = K_chemicals
+        self.Ks = Ks
+        self.top_solvents = top_solvents
+        self.top_split = top_split
+        self.bot_solvents = bot_solvents
+        self.bot_split = bot_split
+
+    def _run(self):
+        feed = self.ins[0]
+        top, bot = self.outs
+        indices = self.chemicals.get_index
+        def flattend(indices, split):
+            flat_index = []
+            flat_split = []
+            integer = int
+            isa = isinstance
+            for i, j in zip(indices, split):
+                if isa(i, integer): 
+                    flat_index.append(i)
+                    flat_split.append(j)
+                else:
+                    flat_index.extend(i)
+                    flat_split.extend([j] * len(i))
+            return flat_index, np.array(flat_split)
+        
+        K_index, Ks = flattend(indices(self.K_chemicals), self.Ks)
+        top_index, top_split = flattend(indices(self.top_solvents), self.top_split)
+        bot_index, bot_split = flattend(indices(self.bot_solvents), self.bot_split)
+        top_mol = top.mol; bot_mol = bot.mol; feed_mol = feed.mol
+        top_mol[top_index] = feed_mol[top_index] * top_split
+        bot_mol[top_index] = feed_mol[top_index] - top_mol[top_index]
+        bot_mol[bot_index] = feed_mol[bot_index] * bot_split
+        top_mol[bot_index] = feed_mol[bot_index] - bot_mol[bot_index]
+        topnet = top_mol[top_index].sum()
+        botnet = bot_mol[bot_index].sum()
+        molnet = topnet+botnet
+        top_mol[K_index] = Ks * topnet * feed_mol[K_index] / molnet  # solvent * mol ratio
+        bot_mol[K_index] = feed_mol[K_index] - top_mol[K_index]
+        top.T, top.P = feed.T, feed.P
+        bot.T, bot.P = feed.T, feed.P
 
 
 class LiquidsSplitCentrifuge(LiquidsCentrifuge):
@@ -222,7 +257,7 @@ class LiquidsSplitCentrifuge(LiquidsCentrifuge):
     
     """
     line = 'Liquids centrifuge'
-    __init__ = Splitter.__init__
+    _init = Splitter._init
     _run = Splitter._run
     split = Splitter.split
     isplit = Splitter.isplit
@@ -326,21 +361,21 @@ class SLLECentrifuge(Unit):
     >>> C1.show()
     SLLECentrifuge: C1
     ins...
-    [0] feed
+    [0] feed  
         phase: 'l', T: 298.15 K, P: 101325 Pa
         flow (kmol/hr): Water   100
                         Hexane  100
                         Solids  10
     outs...
-    [0] oil
+    [0] oil  
         phase: 'l', T: 298.15 K, P: 101325 Pa
         flow (kmol/hr): Water   0.791
                         Hexane  100
-    [1] aqueous
+    [1] aqueous  
         phase: 'l', T: 298.15 K, P: 101325 Pa
         flow (kmol/hr): Water   98.7
                         Hexane  0.015
-    [2] solids
+    [2] solids  
         phase: 'l', T: 298.15 K, P: 101325 Pa
         flow (kmol/hr): Water   0.555
                         Solids  10
@@ -368,11 +403,8 @@ class SLLECentrifuge(Unit):
     def solids_isplit(self):
         return self._solids_isplit
     
-    def __init__(self, ID='', ins=None, outs=(), thermo=None, *,
-                 solids_split, top_chemical=None, efficiency=1.0,
-                 moisture_content=0.5):
-        Unit.__init__(self, ID, ins, outs, thermo)
-        
+    def _init(self, solids_split, top_chemical=None, efficiency=1.0,
+              moisture_content=0.5):
         # [ChemicalIndexer] Splits to 0th outlet stream.
         self._solids_isplit = self.thermo.chemicals.isplit(solids_split)
         
@@ -488,10 +520,7 @@ class SolidLiquidsSplitCentrifuge(Unit):
     def aqueous_isplit(self):
         return self._aqueous_isplit
     
-    def __init__(self, ID='', ins=None, outs=(), thermo=None, *,
-                 aqueous_split, solids_split, moisture_content=0.5):
-        Unit.__init__(self, ID, ins, outs, thermo)
-        
+    def _init(self, aqueous_split, solids_split, moisture_content=0.5):
         # [ChemicalIndexer] Splits to 1st outlet stream (aqueous/heavy phase).
         self._aqueous_isplit = self.thermo.chemicals.isplit(aqueous_split)
         
@@ -506,7 +535,7 @@ class SolidLiquidsSplitCentrifuge(Unit):
         oil, aqueous, solids = self.outs
         self.ins[0].split_to(aqueous, oil, self.aqueous_split)
         aqueous.split_to(solids, aqueous, self.solids_split)
-        sep.adjust_moisture_content(solids, aqueous, self.moisture_content)
+        sep.adjust_moisture_content(solids, aqueous, self.moisture_content, strict=False)
         
         
 
@@ -544,11 +573,8 @@ class LiquidsMixingTank(bst.Unit, PressureVessel):
     _N_ins = 3
     _N_outs = 1
     
-    def __init__(self, ID="", ins=None, outs=(), thermo=None, *,
-                 tau=0.022, agitator_kW_per_m3=1.0, length_to_diameter=1,
-                 vessel_material='Carbon steel',
-                 vessel_type='Vertical'):
-        bst.Unit.__init__(self, ID, ins, outs, thermo)
+    def _init(self, tau=0.022, agitator_kW_per_m3=1.0, length_to_diameter=1,
+              vessel_material='Carbon steel', vessel_type='Vertical'):
         self.length_to_diameter = length_to_diameter
         self.vessel_material = vessel_material
         self.vessel_type = vessel_type
@@ -605,12 +631,10 @@ class LiquidsSettler(bst.Unit, PressureVessel, isabstract=True):
     _N_ins = 1
     _N_outs = 2
     
-    def __init__(self, ID='', ins=None, outs=(), thermo=None, *,
-                 area_to_feed=0.1, 
-                 length_to_diameter=4,
-                 vessel_material='Carbon steel',
-                 vessel_type='Horizontal'):
-        bst.Unit.__init__(self, ID, ins, outs, thermo)
+    def _init(self, area_to_feed=0.1, 
+              length_to_diameter=4,
+              vessel_material='Carbon steel',
+              vessel_type='Horizontal'):
         self.vessel_material = vessel_material
         self.vessel_type = vessel_type
         self.length_to_diameter = length_to_diameter #: Length to diameter ratio
@@ -665,16 +689,16 @@ class LLESettler(LLEUnit, LiquidsSettler):
     
     """
     line = 'Settler'
-    def __init__(self, ID='', ins=None, outs=(), thermo=None, *,
-                 area_to_feed=0.1, 
-                 length_to_diameter=4,
-                 vessel_material='Carbon steel',
-                 vessel_type='Horizontal',
-                 top_chemical=None,
-                 efficiency=1.0,
-                 cache_tolerance=1e-6,
+    def _init(self, 
+            area_to_feed=0.1, 
+            length_to_diameter=4,
+            vessel_material='Carbon steel',
+            vessel_type='Horizontal',
+            top_chemical=None,
+            efficiency=1.0,
+            cache_tolerance=1e-6,
         ):
-        LLEUnit.__init__(self, ID, ins, outs, thermo, top_chemical, efficiency)
+        LLEUnit._init(self, top_chemical, efficiency)
         self.vessel_material = vessel_material
         self.vessel_type = vessel_type
         self.length_to_diameter = length_to_diameter
@@ -710,13 +734,13 @@ class LiquidsSplitSettler(LiquidsSettler):
     
     """
     line = 'Settler'
-    def __init__(self, ID='', ins=None, outs=(), thermo=None, *,
-                 split, order=None,
-                 area_to_feed=0.1, 
-                 length_to_diameter=4,
-                 vessel_material='Carbon steel',
-                 vessel_type='Horizontal'):
-        bst.Unit.__init__(self, ID, ins, outs, thermo)
+    def _init(self, 
+            split, order=None,
+            area_to_feed=0.1, 
+            length_to_diameter=4,
+            vessel_material='Carbon steel',
+            vessel_type='Horizontal'
+        ):
         self.vessel_material = vessel_material
         self.vessel_type = vessel_type
         self.length_to_diameter = length_to_diameter
@@ -750,24 +774,32 @@ class LiquidsPartitionSettler(LiquidsSettler):
     partition_coefficients : 1d array, optional
         Partition coefficients of chemicals in equilibrium (molar 
         composition ratio of the top fluid over the bottom fluid). 
-    partition_IDs: tuple[str], optional
+    partition_IDs : tuple[str], optional
         IDs of chemicals in equilibrium.
-    
+    top_chemicals : tuple[str], optional
+        Chemicals that remain in the top fluid.
+    bottom_chemicals : tuple[str], optional
+        Chemicals that remain in the bottom fluid.
     """
     line = 'Settler'
-    def __init__(self, ID='', ins=None, outs=(), thermo=None, *,
-                 partition_coefficients, partion_IDs, 
-                 area_to_feed=0.1, 
-                 length_to_diameter=4,
-                 vessel_material='Carbon steel',
-                 vessel_type='Horizontal'):
-        bst.Unit.__init__(self, ID, ins, outs, thermo)
+    
+    def _init(self, 
+            partition_coefficients, partition_IDs, 
+            area_to_feed=0.1, 
+            length_to_diameter=4,
+            vessel_material='Carbon steel',
+            vessel_type='Horizontal',
+            top_chemicals=None,
+            bottom_chemicals=None,
+        ):
         self.vessel_material = vessel_material
         self.vessel_type = vessel_type
         self.length_to_diameter = length_to_diameter
         self.area_to_feed = area_to_feed
         self.partition_coefficients = partition_coefficients
-        self.partion_IDs = partion_IDs
+        self.partition_IDs = partition_IDs
+        self.top_chemicals = top_chemicals
+        self.bottom_chemicals = bottom_chemicals
         self.reset_cache()
     
     def reset_cache(self, isdynamic=None):
@@ -775,8 +807,8 @@ class LiquidsPartitionSettler(LiquidsSettler):
     
     def _run(self):
         self._phi = sep.partition(*self.ins, *self.outs, 
-                                  self.partion_IDs, self.partition_coefficients,
-                                  self._phi)
+                                  self.partition_IDs, self.partition_coefficients,
+                                  self._phi, self.top_chemicals, self.bottom_chemicals)
 
 # %% Mixer-settlers
 
@@ -793,8 +825,8 @@ class MixerSettler(bst.Unit):
     outs : 
         * [0] extract.
         * [1] raffinate.
-    solvent_ID : str, optional
-        Name of main chemical in the solvent.
+    top_chemical : str, optional
+        Name of main chemical in the extract phase.
         Defaults to chemical with highest molar fraction in the solvent.
     mixer_data : dict, optional
         Arguments to initialize the "mixer" attribute, a :class:`~biosteam.units.LiquidsMixingTank` object.
@@ -812,9 +844,9 @@ class MixerSettler(bst.Unit):
     >>> MS1 = bst.MixerSettler('MS1', ins=(feed, solvent), outs=('extract', 'raffinate'))
     >>> MS1.simulate()
     >>> MS1.extract.imol['Methanol'] / MS1.feed.imol['Methanol']
-    0.66
+    0.665
     >>> MS1.raffinate.imol['Water'] / MS1.feed.imol['Water']
-    0.82
+    0.827
     >>> MS1.extract.imol['Octanol'] / MS1.solvent.imol['Octanol']
     0.99
     >>> MS1.results() # doctest: +SKIP
@@ -853,7 +885,7 @@ class MixerSettler(bst.Unit):
     ...    model='partition coefficients',
     ...    settler_data={
     ...        'partition_coefficients': np.array([1.451e-01, 1.380e+00, 2.958e+03]),
-    ...        'partion_IDs': ('Water', 'Methanol', 'Octanol'),
+    ...        'partition_IDs': ('Water', 'Methanol', 'Octanol'),
     ...    },
     ... )
     >>> MS1.simulate()
@@ -899,10 +931,7 @@ class MixerSettler(bst.Unit):
     for i,j in LiquidsSettler._units.items(): 
         _units['Settler - ' + i] = j
     
-    def __init__(self, ID='', ins=None, outs=(), thermo=None, 
-                 solvent_ID=None, mixer_data={}, settler_data={}, model="LLE"):
-        bst.Unit.__init__(self, ID, ins, outs, thermo)
-        
+    def _init(self, top_chemical=None, mixer_data={}, settler_data={}, model="LLE"):
         #: [LiquidsMixingTank] Mixer portion of the mixer-settler.
         #: All data and settings for the design of the mixing tank are stored here.
         self.mixer = mixer = LiquidsMixingTank(None, None, (None,),
@@ -922,7 +951,7 @@ class MixerSettler(bst.Unit):
         self.settler = Settler(None, multi_stream, thermo=self.thermo, **settler_data)
         
         #: [str] ID of carrier component in the feed.
-        self.solvent_ID = solvent_ID 
+        self.top_chemical = top_chemical 
     
     @property
     def feed(self):
@@ -958,7 +987,7 @@ class MixerSettler(bst.Unit):
 
     def _run(self):
         self.mixer._run()
-        self.settler.solvent = self.solvent_ID or self.solvent.main_chemical
+        self.settler.top_chemical = self.top_chemical or self.solvent.main_chemical
         self.settler._run()
         for i, j in zip([self.extract, self.raffinate], self.settler.outs): i.copy_like(j)
         
@@ -997,8 +1026,8 @@ class MultiStageMixerSettlers(MultiStageEquilibrium):
         composition ratio of the extract over the raffinate). If given,
         The mixer-settlers will be modeled with these constants. Otherwise,
         partition coefficients are computed based on temperature and composition.
-    solvent_ID : str
-        Name of main chemical in the solvent.
+    top_chemical : str
+        Name of main chemical in the top phase (extract phase).
     mixer_data : dict
         Arguments to initialize the "mixer" attribute, a :class:`~biosteam.units.LiquidsMixingTank` object.
     settler_data : dict
@@ -1092,6 +1121,11 @@ class MultiStageMixerSettlers(MultiStageEquilibrium):
     Because octanol and water do not mix well, it may be a good idea to assume
     that these solvents do not mix at all:
         
+    >>> import biosteam as bst
+    >>> import numpy as np
+    >>> bst.settings.set_thermo(['Water', 'Methanol', 'Octanol'])
+    >>> feed = bst.Stream('feed', Water=5000, Methanol=500)
+    >>> solvent = bst.Stream('solvent', Octanol=5000)
     >>> MSMS1 = bst.MultiStageMixerSettlers('MSMS1', ins=(feed, solvent), outs=('extract', 'raffinate'), N_stages=20,
     ...     partition_data={
     ...         'K': np.array([1.38]),
@@ -1110,6 +1144,11 @@ class MultiStageMixerSettlers(MultiStageEquilibrium):
        
     Simulate with a feed at the 4th stage:
     
+    >>> import biosteam as bst
+    >>> import numpy as np
+    >>> bst.settings.set_thermo(['Water', 'Methanol', 'Octanol'])
+    >>> feed = bst.Stream('feed', Water=5000, Methanol=500)
+    >>> solvent = bst.Stream('solvent', Octanol=5000)
     >>> dilute_feed = bst.Stream('dilute_feed', Water=100, Methanol=2)
     >>> MSMS1 = bst.MultiStageMixerSettlers('MSMS1', ins=(feed, dilute_feed, solvent), outs=('extract', 'raffinate'), N_stages=5,
     ...     feed_stages=[0, 3, -1], # Stage at which each inlet enters, respectively
@@ -1125,7 +1164,12 @@ class MultiStageMixerSettlers(MultiStageEquilibrium):
     0.93
     
     Simulate with a 60% extract side draw at the 2nd stage and 10% raffinate side draw at the 3rd stage:
-    
+
+    >>> import biosteam as bst
+    >>> import numpy as np
+    >>> bst.settings.set_thermo(['Water', 'Methanol', 'Octanol'])
+    >>> feed = bst.Stream('feed', Water=5000, Methanol=500)
+    >>> solvent = bst.Stream('solvent', Octanol=5000)
     >>> MSMS1 = bst.MultiStageMixerSettlers('MSMS1', ins=(feed, solvent), N_stages=4,  
     ...     # Extract side draws always come first, then raffinate side draws
     ...     outs=('extract', 'raffinate', 'extract_side_draw', 'raffinate_side_draw'),  
@@ -1140,22 +1184,21 @@ class MultiStageMixerSettlers(MultiStageEquilibrium):
     ... )
     >>> MSMS1.simulate()
     >>> (MSMS1.extract.imol['Methanol'] + MSMS1.outs[2].imol['Methanol']) / feed.imol['Methanol'] # Recovery
-    1.0
+    0.87
     
-
     """
+    _side_draw_names = ('extract_side_draws', 'raffinate_side_draws')
     _units = MixerSettler._units
+    default_maxiter = 20
     
-    def __init__(self, ID="", ins=None, outs=(), thermo=None, *, N_stages,
-                 partition_data=None, solvent_ID=None, feed_stages=None, 
-                 raffinate_side_draws=None, extract_side_draws=None,
-                 mixer_data={}, settler_data={}, use_cache=None):
-        bst.MultiStageEquilibrium.__init__(
-            self, ID, ins, outs, thermo,
-            N_stages=N_stages, feed_stages=feed_stages, phases=('l', 'L'), P=101325,
+    def _init(self, N_stages, feed_stages=None, extract_side_draws=None, 
+              raffinate_side_draws=None, partition_data=None, top_chemical=None,  
+              mixer_data={}, settler_data={}, use_cache=None, collapsed_init=None):
+        bst.MultiStageEquilibrium._init(
+            self, N_stages=N_stages, feed_stages=feed_stages, phases=('l', 'L'), P=101325,
             top_side_draws=extract_side_draws, bottom_side_draws=raffinate_side_draws,
             stage_specifications=None, partition_data=partition_data, 
-            solvent=solvent_ID, use_cache=use_cache,
+            top_chemical=top_chemical, use_cache=use_cache, collapsed_init=collapsed_init,
         )
         #: [LiquidsMixingTank] Used to design all mixing tanks. 
         #: All data and settings for the design of mixing tanks are stored here.
@@ -1170,12 +1213,14 @@ class MultiStageMixerSettlers(MultiStageEquilibrium):
         self.use_cache = use_cache
         self._last_args = (
             self.N_stages, self.feed_stages, self.extract_side_draws, self.use_cache,
-            *self._ins, self.raffinate_side_draws, self.solvent_ID, self.partition_data
+            *self._ins, self.raffinate_side_draws, self.top_chemical, self.partition_data,
+            self.collapsed_init,
         )
-        self.reset_cache()
     
     feed = MixerSettler.feed
     solvent = MixerSettler.solvent
+    extract = MixerSettler.extract
+    raffinate = MixerSettler.raffinate
     
     @property
     def partition_data(self):
@@ -1188,14 +1233,17 @@ class MultiStageMixerSettlers(MultiStageEquilibrium):
     def _setup(self):
         super()._setup()
         args = (self.N_stages, self.feed_stages, self.extract_side_draws, self.use_cache,
-                *self._ins, self.raffinate_side_draws, self.solvent_ID, self.partition_data)
+                *self._ins, self.raffinate_side_draws, self.top_chemical, self.partition_data,
+                self.collapsed_init)
         if args != self._last_args:
-            MultiStageEquilibrium.__init__(
-                self, self.ID, self.ins, self.outs, self.thermo,
-                N_stages=self.N_stages, feed_stages=self.feed_stages, phases=('l', 'L'), P=self.P,
-                top_side_draws=self.extract_side_draws, bottom_side_draws=self.raffinate_side_draws,
+            MultiStageEquilibrium._init(
+                self, N_stages=self.N_stages, feed_stages=self.feed_stages,
+                phases=('l', 'L'), P=self.P,
+                top_side_draws=self.extract_side_draws, 
+                bottom_side_draws=self.raffinate_side_draws,
                 stage_specifications=None, partition_data=self.partition_data, 
-                solvent=self.solvent_ID, use_cache=self.use_cache,
+                top_chemical=self.top_chemical, use_cache=self.use_cache, 
+                collapsed_init=self.collapsed_init,
             )
             self.mixer._ins = self._ins
             self.settler._outs = self._outs
